@@ -1,15 +1,19 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
 
-import semver
 import re
 import asyncio
 import time
 
-from .sensor import Temperature
-from .cover import Cover
+from typing import Optional, Any, Dict
+
 from .air_quality import AirQuality
-from .light import Light
+from .box_types import default_api_level, get_conf, get_conf_set
+from .button import Button
 from .climate import Climate
+from .cover import Cover
+from .light import Light
+from .sensor import Temperature
+from .session import ApiHost
 from .switch import Switch
 
 from .error import (
@@ -22,19 +26,32 @@ from .error import (
     BadFieldNotANumber,
     BadFieldNotAString,
     BadFieldNotRGBW,
+    HttpError,
 )
 
+
 DEFAULT_PORT = 80
 
 
 class Box:
-    # TODO: pass IP? (For better error messages).
-    def __init__(self, api_session, info):
+    _name: str
+    _data_path: str
+    _last_real_update: Optional[float]
+    api_session: ApiHost
+    info: dict
+    config: dict
+    extended_state: Optional[Dict[Any, Any]]
+
+    def __init__(
+        self,
+        api_session,
+        info,
+        config,
+        extended_state,
+    ) -> None:
         self._last_real_update = None
         self._sem = asyncio.BoundedSemaphore()
         self._session = api_session
-        self._name = "(unnamed)"
-        self._data_path = None
 
         address = f"{api_session.host}:{api_session.port}"
 
@@ -57,6 +74,13 @@ class Box:
         except KeyError:
             product = type
 
+        # TODO: make wLightBox API support multiple products
+        # in 2020 wLightBoxS API has been deprecated and it started using wLightBox API
+        # current codebase needs a refactor to support multiple product sharing one API
+        # as a temporary workaround we are using 'alias' type wLightBoxS2
+        if type == "wLightBox" and product == "wLightBoxS":
+            type = "wLightBoxS"
+
         location = f"{product}:{unique_id} at {address}"
 
         try:
@@ -80,191 +104,159 @@ class Box:
                 info, f"{location} has no hardware version"
             ) from ex
 
-        try:
-            level = int(info["apiLevel"])
-        except KeyError:
-            if type != "gateBox":
-                raise UnsupportedBoxResponse(info, f"{location} has no apiLevel")
+        level = int(info.get("apiLevel", default_api_level))
 
-            level = None
-
-        if type == "switchBox":
-            if level < 20190808:
-                type = "switchBox0"
-
-        # Here due to circular dependency
-        from .products import Products
-
-        try:
-            config = Products.CONFIG["types"][type]
-        except KeyError as ex:
-            raise UnsupportedBoxResponse(
-                info, f"{location} is not a supported type"
-            ) from ex
-
-        # TODO: make wLightBox API support multiple products
-        # in 2020 wLightBoxS API has been deprecated and it started using wLightBox API
-        # current codebase needs a refactor to support multiple product sharing one API
-        # as a temporary workaround we are using 'alias' type wLightBoxS2
-        if type == "wLightBox" and product == "wLightBoxS":
-            config = Products.CONFIG["types"]["wLightBoxS2"]
-            type = "wLightBoxS"
-
-        # Currently code doesn't support different apis for the same types or products
-        # and should be heavily refactored to support that functionality.
-        if type == "gateBox" and level:
-            _min, _max = Products.CONFIG["types"]["gateBoxB"]["api_level_range"]
-            if _min <= level <= _max:
-                config = Products.CONFIG["types"]["gateBoxB"]
-                type = "gateBoxB"
-
-        # Ok to crash here, since it's a bug
         self._data_path = config["api_path"]
-        min_supported, max_supported = config["api_level_range"]
-
-        # TODO: assume all are supported
-        if level is None:
-            level = min_supported
-
-        if level < min_supported:
-            raise UnsupportedBoxVersion(
-                info,
-                f"{location} has outdated firmware (last supported: {min_supported} vs {level}) .",
-            )
-
-        version, outdated = self.extract_version(
-            type, info, min_supported, max_supported, level
-        )
-
         self._type = type
-        self._product = product
         self._unique_id = unique_id
         self._name = name
         self._firmware_version = firmware_version
         self._hardware_version = hardware_version
         self._api_version = level
-        self._version = version
-        self._outdated = outdated
+
         self._model = config.get("model", type)
 
         self._api = config.get("api", {})
 
-        self._features = {}
-        for item in {
+        self._features = self.create_features(config, info, extended_state)
+
+        self._config = config
+
+        self._update_last_data(None)
+
+    def create_features(
+        self, config: dict, info: dict, extended_state: Optional[dict]
+    ) -> dict:
+
+        features = {}
+        for field, klass in {
             "air_qualities": AirQuality,
             "covers": Cover,
             "sensors": Temperature,  # TODO: too narrow
             "lights": Light,
             "climates": Climate,
             "switches": Switch,
+            "buttons": Button,
         }.items():
-            field, klass = item
             try:
-                self._features[field] = [
-                    klass(self, *args) for args in config.get(field, [])
-                ]
-            # TODO: fix constructors instead
-            except KeyError as ex:
-                raise UnsupportedBoxResponse(
-                    info, f"{location} failed to initialize: {ex}"
-                )  # from ex
+                if box_type_config := config.get(field):
+                    features[field] = klass.many_from_config(
+                        self,
+                        box_type_config=box_type_config,
+                        extended_state=extended_state,
+                    )
+            except KeyError:
+                raise UnsupportedBoxResponse("Failed to initialize:", info)
+        return features
 
-        self._config = config
+    @classmethod
+    async def async_from_host(cls, api_host: ApiHost) -> Box:
+        try:
+            path = "/api/device/state"
+            data = await api_host.async_api_get(path)
+        except HttpError:
+            path = "/info"
+            data = await api_host.async_api_get(path)
 
-        self._update_last_data(None)
+        info = data.get("device", data)  # type: ignore
+        extended_state = None
+
+        config = cls._match_device_config(info)
+        if extended_state_path := config.get("extended_state_path"):
+            try:
+                extended_state = await api_host.async_api_get(extended_state_path)
+            except (HttpError, KeyError):
+                extended_state = None
+
+        return cls(api_host, info, config, extended_state)
+
+    @classmethod
+    def _match_device_config(cls, info: dict) -> dict:
+        try:
+            device_type = info["type"]
+        except KeyError as ex:
+            raise UnsupportedBoxResponse(info, "Device info has no type key") from ex
+        try:
+            product = info["product"]
+        except KeyError:
+            product = device_type
+        if device_type == "wLightBox" and product == "wLightBoxS":
+            device_type = "wLightBoxS"
+        level = int(info.get("apiLevel", default_api_level))
+        config_set = get_conf_set(device_type)
+        if not config_set:
+            raise UnsupportedBoxResponse(f"{device_type} is not a supported type")
+        config = get_conf(level, config_set)
+        if not config:
+            raise UnsupportedBoxVersion(
+                f"{device_type} has unsupported version ({level})."
+            )
+
+        return config
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
     @property
-    def last_data(self):
+    def last_data(self) -> Optional[Dict[Any, Any]]:
         return self._last_data
 
+    # Used in full_name, track down and refactor.
     @property
-    def type(self):
+    def type(self) -> str:
         return self._type
 
     @property
-    def product(self):
-        return self._product
-
-    @property
-    def unique_id(self):
+    def unique_id(self) -> Any:
         return self._unique_id
 
     @property
-    def firmware_version(self):
+    def firmware_version(self) -> Any:
         return self._firmware_version
 
     @property
-    def hardware_version(self):
+    def hardware_version(self) -> Any:
         return self._hardware_version
 
     @property
-    def api_version(self):
+    def api_version(self) -> int:
         return self._api_version
 
     @property
-    def version(self):
-        return self._version
-
-    @property
-    def features(self):
+    def features(self) -> dict:
         return self._features
 
     @property
-    def outdated(self):
-        # TODO: fixme
-        return self._outdated
-
-    @property
-    def brand(self):
+    def brand(self) -> str:
         return "BleBox"
 
     @property
-    def model(self):
+    def model(self) -> Any:
         return self._model
 
-    # TODO: report timestamp of last measurement (if possible)
-
-    async def async_update_data(self):
+    async def async_update_data(self) -> None:
         await self._async_api(True, "GET", self._data_path)
 
-    def _update_last_data(self, new_data):
+    def _update_last_data(self, new_data: Optional[dict]) -> None:
         self._last_data = new_data
         for feature_set in self._features.values():
             for feature in feature_set:
                 feature.after_update()
 
-    def extract_version(self, type, device_info, min_supported, max_supported, level):
-        minor = level - min_supported
-
-        current = semver.VersionInfo(1, minor, 0)
-
-        # TODO: uncomment when compatiblity is broken
-        # min_compatibility = config['min_semver'] # e.g. "1.0.0"
-        # if semver.VersionInfo.parse(min_compatibility) > current:
-        #     # TODO: coverage
-        #     raise OutdatedBoxVersion(device_info)
-
-        # TODO: for now, BleBox assumes backward-compatibility
-        max_minor = max_supported - min_supported
-        latest_version = semver.VersionInfo(1, max_minor, 0)
-
-        # TODO: uncomment when backward compatiblity is broken
-        # if current > latest_version:
-        #     raise UnsupportedAppVersion(device_info)
-
-        outdated = current < latest_version
-        return (current, outdated)
-
-    async def async_api_command(self, command, value=None):
+    async def async_api_command(self, command: str, value: Any = None) -> None:
         method, *args = self._api[command](value)
         self._last_real_update = None  # force update
         return await self._async_api(False, method, *args)
 
-    def follow(self, data, path):
+    def follow(self, data: dict, path: str) -> Any:
+        """
+        Return payloadu from device response json.
+        :param self:
+        :param data:
+        :param path:
+        :return:
+        """
         if data is None:
             raise RuntimeError(f"bad argument: data {data}")  # pragma: no cover
 
@@ -339,63 +331,77 @@ class Box:
                     path,
                     data,
                 )
-
         return current_tree
 
-    def expect_int(self, field, raw_value, maximum=-1, minimum=0):
+    def expect_int(
+        self, field: str, raw_value: int, maximum: int = -1, minimum: int = 0
+    ) -> int:
         return self.check_int(raw_value, field, maximum, minimum)
 
-    def expect_hex_str(self, field, raw_value, maximum=-1, minimum=0):
+    def expect_hex_str(
+        self, field: str, raw_value: int, maximum: int = -1, minimum: int = 0
+    ) -> int:
         return self.check_hex_str(raw_value, field, maximum, minimum)
 
-    def expect_rgbw(self, field, raw_value):
+    def expect_rgbw(self, field: str, raw_value: int) -> int:
         return self.check_rgbw(raw_value, field)
 
-    def check_int_range(self, value, field, max, min):
-        if max >= min:
-            if value > max:
-                raise BadFieldExceedsMax(self.name, field, value, max)
-            if value < min:
-                raise BadFieldLessThanMin(self.name, field, value, min)
-
+    def check_int_range(
+        self, value: int, field: str, max_value: int, min_value: int
+    ) -> int:
+        if max_value >= min_value:
+            if value > max_value:
+                raise BadFieldExceedsMax(self.name, field, value, max_value)
+            if value < min_value:
+                raise BadFieldLessThanMin(self.name, field, value, min_value)
         return value
 
-    def check_int(self, value, field, maximum, minimum):
+    def check_int(self, value: int, field: str, max_value: int, min_value: int) -> int:
         if value is None:
             raise BadFieldMissing(self.name, field)
 
         if not type(value) is int:
             raise BadFieldNotANumber(self.name, field, value)
 
-        return self.check_int_range(value, field, maximum, minimum)
+        return self.check_int_range(value, field, max_value, min_value)
 
-    def check_hex_str(self, value, field, maximum, minimum):
+    def check_hex_str(
+        self, value: int, field: str, max_value: int, min_value: int
+    ) -> int:
         if value is None:
             raise BadFieldMissing(self.name, field)
 
         if not isinstance(value, str):
             raise BadFieldNotAString(self.name, field, value)
 
-        return self.check_int_range(int(value, 16), field, maximum, minimum)
+        if (
+            self.check_int_range(int(value, 16), field, max_value, min_value)
+            is not None
+        ):
+            return value
 
-    def check_rgbw(self, value, field):
+    def check_rgbw(self, value: int, field: str) -> int:
         if value is None:
             raise BadFieldMissing(self.name, field)
 
         if not isinstance(value, str):
             raise BadFieldNotAString(self.name, field, value)
 
-        # value can have different length depending on LED color mode
-        # mono mode will be 1 byte, and RGBWW will be 5 bytes
         if len(value) > 10 or len(value) % 2 != 0:
             raise BadFieldNotRGBW(self.name, field, value)
         return value
 
-    def _has_recent_data(self):
+    def _has_recent_data(self) -> bool:
         last = self._last_real_update
-        return time.time() - 2 <= last if last is not None else False
+        return (time.time() - 2) <= last if last is not None else False
 
-    async def _async_api(self, is_update, method, path, post_data=None):
+    async def _async_api(
+        self,
+        is_update: bool,
+        method: Any,
+        path: str,
+        post_data: dict = None,
+    ) -> None:
         if method not in ("GET", "POST"):
             raise NotImplementedError(method)  # pragma: no cover
 
@@ -407,7 +413,6 @@ class Box:
             if is_update:
                 if self._has_recent_data():
                     return
-
             if method == "GET":
                 response = await self._session.async_api_get(path)
             else:
