@@ -1,24 +1,32 @@
 from .feature import Feature
+from typing import TYPE_CHECKING, Union
+
+if TYPE_CHECKING:
+    from .box import Box
 
 
 class Sensor(Feature):
+    _unit: str
+
     @property
-    def unit(self):
+    def unit(self) -> str:
         return self._unit
 
 
 class Temperature(Sensor):
-    def __init__(self, product, alias, methods):
+    _current: Union[float, int, None]
+
+    def __init__(self, product: "Box", alias: str, methods: dict):
         self._unit = "celsius"
         self._device_class = "temperature"
         super().__init__(product, alias, methods)
 
     @property
-    def current(self):
+    def current(self) -> Union[float, int, None]:
         return self._current
 
     # TODO: use as attribute in product config
-    def _read_temperature(self, field):
+    def _read_temperature(self, field: str) -> Union[float, int, None]:
         product = self._product
         if product.last_data is not None:
             raw = self.raw_value(field)
@@ -27,5 +35,5 @@ class Temperature(Sensor):
                 return round(product.expect_int(alias, raw, 12500, -5500) / 100.0, 1)
         return None
 
-    def after_update(self):
+    def after_update(self) -> None:
         self._current = self._read_temperature("temperature")
